"""Module containing useful resources that are used, for example, for data annotation in azol"""
from .graph_delegated_permissions import delegated_permissions
from .graph_permissions_map import graph_permission_map
from .rbac_roles import rbac_roles
